@@ -47,19 +47,6 @@
 #define POOLKEYFILE 	".pgpoolkey"
 #define POOLKEYFILEENV "PGPOOLKEYFILE"
 
-/*
- * Brought from PostgreSQL's pg_config_manual.h.
- *
- * Maximum length for identifiers (e.g. table names, column names,
- * function names).  Names actually are limited to one less byte than this,
- * because the length must include a trailing zero byte.
- *
- * Please note that in version 2 protocol, maximum user name length is
- * SM_USER, which is 32.
- */
-#define NAMEDATALEN 64
-
-/* configuration file name */
 #define POOL_CONF_FILE_NAME "pgpool.conf"
 
 /* PCP user/password file name */
@@ -356,9 +343,9 @@ extern int	my_main_node_id;
 typedef enum
 {
 	POOL_NODE_STATUS_UNUSED,	/* unused */
-	POOL_NODE_STATUS_PRIMARY,	/* primary ndoe */
+	POOL_NODE_STATUS_PRIMARY,	/* primary node */
 	POOL_NODE_STATUS_STANDBY,	/* standby node */
-	POOL_NODE_STATUS_INVALID	/* invalid node (split branin, stand alone) */
+	POOL_NODE_STATUS_INVALID	/* invalid node (split brain, stand alone) */
 }			POOL_NODE_STATUS;
 
 /* Clustering mode macros */
@@ -509,7 +496,7 @@ typedef enum
 }			POOL_RECOVERY_MODE;
 
 /*
- * Process types.  DO NOT change the order of each enum meber!  If you do
+ * Process types.  DO NOT change the order of each enum member!  If you do
  * that, you must change application_name array in src/main/pgpool_main.c
  * accordingly.
  */
